#!/usr/bin/env python

import argparse
import os
import numpy as np
import iio
import re

np.warnings.filterwarnings('ignore')

parser = argparse.ArgumentParser()
parser.add_argument('input', type=str, help=('path to source (tiff)'))
parser.add_argument('output', type=str, help=('path to output (tiff)'))
parser.add_argument('chsim', type=str, help=(''))
parser.add_argument('chano', type=str, help=(''))
parser.add_argument('log', type=bool, help=('apply a log to the input'))

args = parser.parse_args()

img = iio.read(args.input)


if args.log:
    img = np.log(img)

# Normalize variance of all channels
std = np.std(img, axis=(0,1), keepdims=True)
img = img/std

visu = img[:,:,0]
visu = (visu - np.percentile(visu.flatten(), 5))/np.percentile(visu.flatten(), 90) * 255.
iio.write("input_0.png", visu)

# Clean input strings
chsim = re.sub("[^0-9 \+\/\-\*\(\)]", "", args.chsim)
chsim = re.sub(" +", " ", chsim)
if chsim[0] == ' ':
    chsim = chsim[1:]
if chsim[-1] == ' ':
    chsim = chsim[:-1]
chano = re.sub("[^0-9 \+\/\-\*\(\)]", "", args.chano)
chano = re.sub(" +", " ", chano)
if chano[0] == ' ':
    chano = chano[1:]
if chano[-1] == ' ':
    chano = chano[:-1]

# Reorganize the channels according to chsim and chano
chsim = chsim.split(' ')
chano = chano.split(' ')

# Drop channels that are not in chsim or chano, then first are
# the chsim channels then the channels that are in both lists and
# finally the chano channels
order = {}

for i in chsim:
    order[i] = 1

for i in chano:
    if i in order:
        order[i] = 2
    else:
        order[i] = 3

# Remove useless channels
order = [(k,order[k]) for k in order]
# Sort the channels according to the priority
order = sorted(order, key=lambda a: a[1])
# Write info regarding the order of the channels
chsimf = open("dimsim.txt","w") 
numch = 0
for a in order:
    if a[1] < 3:
        numch += 1 
chsimf.write(str(numch)) 
chsimf.close()
chanof = open("dimano.txt","w") 
numch = 0
for a in order:
    if a[1] == 1:
        numch += 1 
chanof.write(str(numch)) 
chanof.close()

# Keep only the channel order
chorder = [a[0] for a in order]

# Write the info about the number of channels for visualization
algo_info = open("algo_info.txt","w") 
algo_info.write('nchannels='+str(len(order))) 
algo_info.close()

# Write the order of channel
algo_info = open("desc_channels.txt","w") 
for i in range(len(order)):
    algo_info.write(str(i) + ": " + chorder[i]+'\n')
algo_info.close()

out = np.zeros((img.shape[0], img.shape[1], len(chorder)))
for i in range(len(chorder)):
    comb = [j for j in re.split(r'(\d+)', chorder[i])]
    comb = ''.join(['img[:,:,'+j+'-1]' if j.isdigit() else j for j in comb])
    out[:,:,i] = eval(comb)

# Removes NaNs if any
out[np.logical_not(np.isfinite(out))] = 0

iio.write(args.output, out)
